;===========================================================================
;
;  ZTris 82 (by Tijl Coosemans, tijl@ulyssis.org)
;  ~~~~~~~~
;================================= version 1.3 == 02/28/2002 (MM/DD/YYYY) ==
.INCLUDE ACEBETA.INC
#DEFINE RomCall(label) call label
.nolist
;===========================================================================
_clrScrnFull		=CLEAR_TEXT
_divHLby10		=UNPACK_HL
_getcsc			=GET_KEY
_putc			=TX_CHARPUT
_putmap			=TX_CHARPUT ;note: ace does not have TR_CHARPUT
_puts			=D_ZT_STR
_vputs			=D_ZM_STR
_vputmap		=M_CHARPUT
;===========================================================================
currow			=CURSOR_ROW
curcol			=CURSOR_COL
pencol			=CURSOR_X
penrow			=CURSOR_Y
screenBuf		=GRAPH_MEM
;===========================================================================
safeRam			=SH_BUF
decomp			=safeRam
penLines		=safeRam+8
opponentStatus		=safeRam+9
vdispHLStr		=safeRam+10
height			=safeRam+16
sendBuf1		=safeRam+17
sendBuf2		=safeRam+18
sendBuf3		=safeRam+19
counter			=safeRam+20
nxtPceBufOffset		=safeRam+21
;===========================================================================
.list

	.db	"ZTris 1.3",0

;===========================================================================
; * Initialize Game *
;===========================================================================

	ld	a,r
	ld	(randSeed+1),a
	set	7,(iy+20)
restartGame:
	ld	hl,safeRam+15
	xor	a
	ld	b,7
initGameLoop:
	ld	(hl),a
	inc	hl
	djnz	initGameLoop
	ld	hl,resume
	or	(hl)
	jp	nz,drawLevel

;===========================================================================
; * Choose Game Options *
;===========================================================================

gameOptionsChanged:
	call	showTitleScrn
	ld	hl,heightTxt
	ld	de,104Ah
	call	fastVPuts
	ld	de,0311h
	ld	a,(gameOptions)
	or	2
	ld	(852Dh),a
	call	fastVPuts
	xor	8
	ld	(852Dh),a
	ld	e,38h
	call	fastVPuts
	res	1,(iy+5)
	res	3,(iy+5)
	bit	3,a
	jr	nz,twoPlayerOptions
dispHighScores:
	ld	hl,highScoreTable
	ld	de,2312h
	ld	c,3
dispHighScoresLoop:
	push	de
	call	fastVPuts
	push	hl
	call	_ldHLind
	ld	e,70
	ld	b,5
	call	vdispHL
	pop	hl
	inc	hl
	inc	hl
	pop	de
	ld	a,d
	add	a,6
	ld	d,a
	dec	c
	jr	nz,dispHighScoresLoop
	jr	dispLevel
twoPlayerOptions:
	ld	de,2D11h
	ld	bc,0202h
twoPlayerOptionsLoop:
	call	fastVPuts
	ld	a,(gameOptions)
	and	c
	ld	d,0
	ld	e,a
	add	hl,de
	RomCall(_vputs)
	ld	de,2511h
	ld	c,4
	djnz	twoPlayerOptionsLoop
	ld	hl,linesTxt
	RomCall(_vputs)
dispLevel:
	call	fastCopy
levelChanged:
	ld	hl,levelsTxt
	ld	de,0202h
	call	fastPuts
	inc	e
	call	fastPuts
	ld	a,(stLevel)
	ld	b,a
	ld	l,2
	cp	5
	jr	c,dispLevelFirstRow
	inc	l
	sub	5
dispLevelFirstRow:
	add	a,a
	add	a,3
	ld	h,a
	ld	a,b
	set	3,(iy+5)
	call	fastPutDigit
	res	3,(iy+5)
heightChanged:
	ld	hl,0E03h
	ld	a,(stHeight)
	call	fastPutDigit
optionsGetKey:
	call	linkReceive
	call	_getcsc
	halt
	ld	hl,stLevel
	dec	a
	jr	z,levelUpDown
	dec	a
	jr	z,decLevel
	dec	a
	jr	z,incLevel
	dec	a
	jr	z,levelUpDown
	ld	hl,stHeight
	sub	49-4
	jr	z,incHeight
	dec	a
	jr	z,decHeight
	ld	hl,gameOptions
	dec	a
	jr	z,changeScrmLines
	dec	a
	jr	z,changeSendLines
	dec	a
	jr	z,changePlayers
	dec	a
	jr	z,playGame
	cp	9-54
	jr	z,playGame
	cp	15-54
	jr	nz,optionsGetKey
	ret
levelUpDown:
	ld	a,(hl)
	add	a,5
changeLevel:
	daa		; modulo 10
	and	0Fh	;
	ld	(hl),a
	jr	levelChanged
incLevel:
	ld	a,(hl)
	inc	a
	jr	changeLevel
decLevel:
	ld	a,(hl)
	add	a,9
	jr	changeLevel
incHeight:
	ld	a,(hl)
	cp	5
	jr	z,optionsGetKey
	inc	a
	jr	changeHeight
decHeight:
	ld	a,(hl)
	or	a
	jr	z,optionsGetKey
	dec	a
changeHeight:
	ld	(hl),a
	jr	heightChanged
changePlayers:
	ld	a,8
	jr	changeGameOptions
changeSendLines:
	ld	a,4
	jr	changeGameOptions
changeScrmLines:
	ld	a,2
changeGameOptions:
	xor	(hl)
	ld	(hl),a
	jp	gameOptionsChanged

;===========================================================================
; * Play Game *
;===========================================================================

playGame:
	call	showTitleScrn
	call	fastCopy
	ld	hl,waitingTxt
	ld	de,0303h
	call	fastPuts
	call	linkSend
	jr	nz,startGame
playWait:
	call	_getcsc
	halt
	or	a
	jp	nz,restartGame
	call	linkReceive
	jr	nc,playWait
startGame:
	call	getNewPce
	call	z,linkSend
	call	getNewPce
	call	z,linkSend
	call	z,linkSend

	ld	hl,gameData
	xor	a
	ld	b,4
initGame:
	ld	(hl),a
	inc	hl
	djnz	initGame
	inc	hl
	inc	hl
	ld	b,19
initBoardLoop:
	ld	(hl),11100000b
	inc	hl
	ld	(hl),00000111b
	inc	hl
	djnz	initBoardLoop
	dec	(hl)
	inc	hl
	ld	a,(hl)
	inc	hl
	ld	(hl),a
	inc	hl
	ld	a,(hl)
	ld	(height),a
drawLevel:
	ld	hl,patterns
	ld	a,(level)
	cp	16
	jr	c,drawPattern
	ld	a,15
drawPattern:
	add	a,a
	add	a,a
	add	a,a
	ld	d,0
	ld	e,a
	add	hl,de
	ld	de,screenBuf
	ld	bc,0808h
drawScreenLoop:
	push	bc
	push	hl
	ld	b,c
drawOneRowLoop:
	ld	a,(hl)
	ld	c,12
drawOneLineLoop:
	ld	(de),a
	inc	de
	dec	c
	jr	nz,drawOneLineLoop
	inc	hl
	djnz	drawOneRowLoop
	pop	hl
	pop	bc
	djnz	drawScreenLoop
drawGaps:
	ld	hl,gaps
	ld	a,4
drawGapsLoop:
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	push	hl
	ex	de,hl
	ld	de,9
makeGap:
	ld	b,3
clearOneLine:
	ld	(hl),d
	inc	hl
	djnz	clearOneLine
	add	hl,de
	dec	c
	jr	nz,makeGap
	pop	hl
	dec	a
	jr	nz,drawGapsLoop
drawBoardLines:
	ld	hl,screenBuf+4
	ld	e,6
	ld	b,64
drawBoardLinesLoop:
	ld	a,(hl)
	or	01h
	ld	(hl),a
	add	hl,de
	ld	a,(hl)
	or	80h
	ld	(hl),a
	add	hl,de
	djnz	drawBoardLinesLoop

	ld	hl,gameInfoTxt
	ld	de,0209h
	call	fastVPuts
	ld	d,18
	call	fastVPuts
	ld	d,34
	call	fastVPuts

	ld	hl,(nxtPce)
	ld	de,0402h
	xor	a
	call	drawPce

	ld	hl,height
	ld	a,(hl)
	ld	(hl),0
	add	a,a
	ld	b,a
	jr	nz,addHeightLines
	jr	chkStatus
addLinesChk:
	ld	hl,penLines
	xor	a
	ld	b,(hl)
	ld	(hl),a
	cp	b
	jr	z,drawStatus
	push	bc
	ld	hl,cPcePos
	ld	a,(hl)
	add	a,b
	cp	16
	jr	c,topNotReached
	ld	a,15
topNotReached:
	ld	(hl),a
	ld	a,b
	add	a,a
	ld	b,0
	ld	c,a
	ld	hl,board+37
	ld	d,h
	ld	e,l
	sbc	hl,bc
	sub	36
	neg
	ld	c,a
	lddr
	pop	bc
	ld	a,(gameOptions)
	bit	1,a
addHeightLines:
	ld	a,10
	call	z,genRandom
	ld	hl,board+2
addLinesLoop:
	push	bc
	push	hl
	ld	de,1111111111111111b
	ld	b,5
addLineGaps:
	push	af
	call	nz,genRandom
	inc	a
	ld	hl,1111111111111011b
rotateWordLoop:
	sl1	l
	rl	h
	dec	a
	jr	nz,rotateWordLoop
	ld	a,d
	and	h
	ld	d,a
	ld	a,e
	and	l
	ld	e,a
	pop	af
	djnz	addLineGaps
	pop	hl
	ld	(hl),d
	inc	hl
	ld	(hl),e
	inc	hl
	pop	bc
	djnz	addLinesLoop

chkStatus:
	ld	hl,board+33
	ld	b,16
chkStatusLoop:
	ld	e,(hl)
	dec	hl
	ld	d,(hl)
	dec	hl
	push	hl
	ld	hl,$E007
	or	a
	sbc	hl,de
	pop	hl
	jr	nz,chkStatusDone
	djnz	chkStatusLoop
chkStatusDone:
	ld	a,b
	or	80h
	ld	(sendBuf1),a

	ld	a,(gameOptions)
	bit	3,a
	jr	nz,dispScore
drawStatus:
	ld	a,(opponentStatus)
	add	a,a
	add	a,a
	ld	de,winTxt
	jp	c,gameWin
	inc	a
	ld	c,a
	ld	a,66
	sub	c
	ld	b,a
	ld	hl,screenBuf+11
	ld	de,12
	jr	drawStatus1
drawStatusLoop1:
	ld	a,(hl)
	and	0Fh
	ld	(hl),a
	add	hl,de
drawStatus1:
	djnz	drawStatusLoop1
	ld	b,c
	jr	drawStatus2
drawStatusLoop2:
	ld	a,(hl)
	and	0Fh
	or	60h
	ld	(hl),a
	add	hl,de
drawStatus2:
	djnz	drawStatusLoop2
dispScore:
	ld	hl,(score+2)
	ld	h,0
	call	_divHLby10
	ld	(score+2),a
	ld	de,(score)
	add	hl,de
	ld	(score),hl
	ld	de,9*256+12
	ld	b,5
	call	vdispHL
	ld	de,25*256+24
	ld	a,(level)
	ld	b,2
	call	vdispA
	ld	de,41*256+20
	ld	a,(lines)
	ld	b,3
	call	vdispA
drawBoard:
	ld	l,16
drawBoardLoop1:
	ld	h,10
drawBoardLoop2:
	push	hl
	ld	a,h
	add	a,10
	ld	h,a
	call	drawBlock
	pop	hl
	dec	h
	jr	nz,drawBoardLoop2
	dec	l
	jr	nz,drawBoardLoop1
	ld	hl,(cPce)
	ld	de,(cPcePos)
	ld	a,(cPceRot)
	push	af
	push	de
	call	drawPce
	call	fastCopy
	pop	de
	pop	af
	call	testPce
	jp	nz,gameOver

;===========================================================================
; * Main Input Loop *
;===========================================================================

inputLoop:
	call	linkReceive
	push	af
	call	linkSendChk
	pop	af
	jp	nz,addLinesChk
	ld	hl,counter
	dec	(hl)
	jr	z,fallDown
	ld	de,(cPcePos)
	ld	a,(cPceRot)
	ld	b,a
	call	_getcsc
	halt
	cp	15
	jp	z,gameOver
	cp	40
	jr	z,drop
	dec	a
	jr	z,moveDown
	dec	a
	jr	z,moveLeft
	dec	a
	jr	z,moveRight
	dec	a
	jr	z,rotateRight
	cp	48-4
	jr	z,rotateLeft
	sub	54-4
	jr	z,rotateRight
	ld	hl,gameOptions
	bit	3,(hl)
	jr	z,inputLoop
	dec	a
	jr	z,pause
	dec	a
	jr	nz,inputLoop
	ret
moveDown:
	ld	hl,score+2
	inc	(hl)
fallDown:
	call	resetCounter
	ld	de,(cPcePos)
	ld	a,(cPceRot)
	dec	e
	call	testPce
	jp	z,dispScore
	jr	newPce
moveLeft:
	dec	d
	dec	d
moveRight:
	inc	d
	inc	b
rotateLeft:
	dec	b
	dec	b
rotateRight:
	inc	b
update:
	call	testPceN
	jp	drawBoard
pause:
	call	showTitleScrn
	call	fastCopy
	ld	hl,pauseTxt
	ld	de,0403h
	call	fastPuts
pauseGetKey:
	call	_getcsc
	halt
	sub	55
	jr	nz,pauseGetKey
	ld	(counter),a
	jp	drawLevel
drop:
	push	bc
	ld	hl,score+2
	inc	(hl)
	dec	e
	call	testPceN
	pop	bc
	jr	z,drop
	call	resetCounter
newPce:
	ld	hl,decomp
	ld	b,4
storePceLoop:
	push	hl
	call	_ldHLind
	ld	de,(cPcePos)
	add	hl,de
	call	getBlock
	or	(hl)
	ld	(hl),a
	pop	hl
	inc	hl
	inc	hl
	djnz	storePceLoop
chkLines:
	ld	hl,board+2
	ld	bc,36*256
chkLinesLoop:
	push	hl
	call	_ldHLind
	inc	hl
	ld	a,h
	or	l
	pop	hl
	jr	nz,nxtRow
	push	bc
	push	hl
	ld	d,h
	ld	e,l
	inc	hl
	inc	hl
	ld	c,b
	ld	b,a	; a = 0
	ldir
	pop	hl
	pop	bc
	inc	c
	jr	chkLinesLoop
nxtRow:	inc	hl
	inc	hl
	dec	b
	djnz	chkLinesLoop
	ld	a,c
	cp	2
	jr	c,nopenalty
	ld	a,(gameOptions)
	rra
	rra
	sub	c
	neg
	ld	(sendBuf2),a
nopenalty:
	ld	hl,scoring
	add	hl,bc
	ld	l,(hl)
	ld	a,(level)
	inc	a
	ld	h,a
	RomCall(_HtimesL)
	ld	de,(score)
	add	hl,de
	ld	(score),hl
	ld	a,(lines)
	add	a,c
	ld	(lines),a
	ld	l,a
	ld	h,0
	call	_divHLby10
	ld	a,l
	ld	hl,level
	cp	(hl)
	jr	c,nonewlvl
	ld	(hl),a
nonewlvl:
	call	getNewPce
	jp	drawLevel

resetCounter:
	ld	hl,levelCnts
	ld	a,(level)
	call	pointHLindA
	ld	(counter),a
	ret

;===========================================================================
; * Game Over *
;===========================================================================

gameOver:
	ld	a,$FF
	ld	(sendBuf1),a
	call	linkReceive
	call	linkSendChk
	ld	de,gameOverTxt
gameWin:
	ld	hl,resume
	inc	(hl)
flashText:
	ld	b,20
flashWait:
	call	_getcsc
	halt
	cp	5
	jr	nc,chkHighScore
	djnz	flashWait
	ld	a,(iy+5)
	xor	8
	ld	(iy+5),a
	push	de
	call	fastCopy
	ld	de,0303h
	pop	hl
	push	hl
	call	fastPuts
	pop	de
	jr	flashText
chkHighScore:
	res	3,(iy+5)
	ld	hl,highScoreTable+14
	ld	b,3
chkHighScoreLoop:
	push	hl
	call	_ldHLind
	ld	de,(score)
	sbc	hl,de
	pop	hl
	jr	c,highScore
	ld	de,16
	add	hl,de
	djnz	chkHighScoreLoop
highScoreDone:
	jp	restartGame
highScore:
	push	de
	ld	a,b
	ld	hl,highScoreTable+31
	ld	de,highScoreTable+47
	jr	moveHighScores
moveHighScoresLoop:
	ld	bc,15
	lddr
	dec	hl
	dec	de
moveHighScores:
	dec	a
	jr	nz,moveHighScoresLoop
	pop	hl
	ex	de,hl
	ld	(hl),d
	dec	hl
	ld	(hl),e
	dec	hl
	ld	b,10
clearHighScore:
	dec	hl
	ld	(hl),' '
	djnz	clearHighScore
	push	hl
	call	showTitleScrn
	ld	hl,highScoreTxt
	ld	de,0B12h
	call	fastVPuts
	ld	de,141Ah
	call	fastVPuts
	call	fastCopy
	ld	hl,0405h
	ld	(currow),hl
	pop	de
	ld	b,0
nameInput:
	call	_getcsc
	halt
	cp	56
	jr	z,deleteChar
	cp	54
	jr	z,highScoreDone
	sub	9
	jr	c,nameInput
	jr	z,highScoreDone
	cp	48-9
	jr	nc,nameInput
	push	de
	ld	hl,characters-1
	call	pointHLindA
	pop	de
	or	a
	jr	z,nameInput
	ld	c,a
	ld	a,b
	cp	10
	jr	z,nameInput
	ld	a,c
	ld	(de),a
	inc	de
	inc	b
	RomCall(_putc)
	jr	nameInput
deleteChar:
	ld	a,b
	or	a
	jr	z,nameInput
	ld	a,' '
	dec	b
	dec	de
	ld	(de),a
	ld	hl,curcol
	dec	(hl)
	RomCall(_putmap)
	jr	nameInput

;===========================================================================
; * Routines *
;===========================================================================

testPceN:
	ld	a,b
testPce:
	push	af
	ld	hl,(cPce)
	call	uncrunch
testPceLoop:
	push	hl
	push	de
	call	_ldHLind
	add	hl,de
	call	getBlock
	and	(hl)
	pop	de
	pop	hl
	jr	nz,testPceDone
	inc	hl
	inc	hl
	djnz	testPceLoop
testPceDone:
	pop	hl
	ret	nz
	ld	a,h
	ld	(cPceRot),a
	ld	(cPcePos),de
	ret

getNewPce:
	ld	hl,(nxtPce)
	ld	(cPce),hl
	ld	hl,14*256+15
	ld	(cPcePos),hl
	xor	a
	ld	(cPceRot),a
	ld	b,a
	ld	hl,nxtPceBufOffset
	cp	(hl)
	push	af
	jr	z,createNewPce
	ld	c,(hl)
	dec	(hl)
	inc	hl
	ld	a,(hl)
	ld	d,h
	ld	e,l
	inc	hl
	ldir
	jr	initNewPce
createNewPce:
	ld	a,7
	call	genRandom
	inc	a
	ld	(sendBuf3),a
initNewPce:
	add	a,a
	add	a,a
	add	a,a
	ld	c,a
	ld	hl,pieces-8
	add	hl,bc
	ld	(nxtPce),hl
	pop	af
	ret

drawPce:
	call	uncrunch
drawPceLoop:
	push	hl
	push	de
	call	_ldHLind
	add	hl,de
	call	putBlock
	pop	de
	pop	hl
	inc	hl
	inc	hl
	djnz	drawPceLoop
	ret

drawBlock:
	push	hl
	call	getBlock
	and	(hl)
	pop	hl
	jr	z,eraseBlock
putBlock:
	call	getScreenLoc
	ret	c
	or	(hl)
	jr	drawBlockLoop
eraseBlock:
	call	getScreenLoc
	ret	c
	cpl
	and	(hl)
drawBlockLoop:
	ld	(hl),a
	add	hl,de
	dec	c
	jr	nz,drawBlockLoop
	ret

getBlock:
	ld	a,l
	add	a,a
	ld	l,a
	ld	a,h
	add	a,240
	jr	nc,getBlock1
	inc	l
getBlock1:
	and	00000111b
	inc	a
	ld	h,a
	ld	a,00000001b
getBlockLoop1:
	rrca
	dec	h
	jr	nz,getBlockLoop1
	ld	de,board
	add	hl,de
	ret

getScreenLoc:		; hl is address of (h,l) in screenBuf
	ld	a,16
	sub	l
	ret	c
	add	a,a
	add	a,a
	ld	l,a
	ld	a,h
	ld	h,0
	ld	d,h
	ld	e,l
	add	hl,de
	add	hl,de
	add	hl,hl
	add	hl,hl
	ld	e,a
	dec	e
	rr	e
	add	hl,de
	ld	de,screenBuf
	add	hl,de
	and	1
	ld	a,00001111b
	ld	c,4
	ld	de,12
	ret	z
	cpl
	ret

uncrunch:		; Extracts the piece from compressed data
	push	de
	and	00000011b
	call	pointHLind2A
	ld	de,decomp
	push	de
	ld	b,8
uncrunchLoop:
	ld	a,l
	and	00000011b
	ld	(de),a
	inc	de
	rr	h
	rr	l
	rr	h
	rr	l
	djnz	uncrunchLoop
	ld	b,4
	pop	hl
	pop	de
	ret

linkReceive:
	di
	in	a,(0)
	and	03h
	cp	03h
	ret	z
	call	receiveByte
	ld	(opponentStatus),a
	call	receiveByte
	ld	(penLines),a
	call	receiveByte
	cp	1
	ret	c
	ld	hl,nxtPceBufOffset
	inc	(hl)
	ld	c,(hl)
	add	hl,bc		; b = 0
	ld	(hl),a
	ret
receiveByte:
	ld	b,8
receiveByteLoop:
	ld	hl,0
receiveBitLoop1:
	dec	hl
	ld	a,h
	or	l
	ret	z
	in	a,(0)
	and	03h
	cp	03h
	jr	z,receiveBitLoop1
	ld	d,2
	cp	d
receiveBit1:
	ld	a,$D4
	jr	nz,receiveBit
receiveBit0:
	ld	a,$E8
	dec	d
receiveBit:
	rr	c
	out	(0),a
	ld	hl,0
receiveBitLoop2:
	dec	hl
	ld	a,h
	or	l
	jr	z,receiveQuit
	in	a,(0)
	and	d
	jr	z,receiveBitLoop2
receiveQuit:
	ld	a,$C0
	out	(0),a
	ret	z
	djnz	receiveByteLoop
	ld	a,c
	ret

linkSend:
	ld	a,80h
	ld	(sendBuf1),a
linkSendChk:
	ld	a,(gameOptions)
	bit	3,a
	ret	nz
	ld	hl,sendBuf1
	ld	a,(hl)
	or	a
	ret	z
	call	sendByte
	call	sendByte
sendByte:
	ld	b,8
	ld	c,(hl)
sendByteLoop:
	rr	c
sendBit0:
	ld	a,$D4
	jr	nc,sendBit
sendBit1:
	ld	a,$E8
sendBit:
	ld	de,0
	out	(0),a
sendBitLoop1:
	dec	de
	ld	a,e
	or	d
	jr	z,sendQuit
	in	a,(0)
	and	03h
	jr	nz,sendBitLoop1
	ld	e,a
sendQuit:
	or	d
	ld	a,$C0
	out	(0),a
	ret	z
	ld	d,e
sendBitLoop2:
	dec	de
	ld	a,d
	or	e
	ret	z
	in	a,(0)
	and	03h
	cp	03h
	jr	nz,sendBitLoop2
	djnz	sendByteLoop
	ld	(hl),b
	inc	hl
	ret

showTitleScrn:
	ld	hl,screenBuf
	ld	de,screenBuf+1
	ld	bc,767
	ld	(hl),0
	ldir
	ld	a,-7
	ld	(penrow),a
	ld	hl,titleTxt
	ld	b,8
showTitle:
	ld	a,(penrow)
	add	a,8
	ld	d,a
	ld	e,2
	call	fastVPuts
	djnz	showTitle
	ld	hl,screenBuf
	ld	de,12
	ld	b,64
invertTitle:
	ld	a,(hl)
	xor	$FE
	ld	(hl),a
	add	hl,de
	djnz	invertTitle
	ret

;===========================================================================
; * Utility Routines *
;===========================================================================

vdispA:	ld	h,0
	ld	l,a

vdispHL:
	push	de
	ld	de,vdispHLStr+5
vdispHLLoop:
	dec	de
	call	_divHLby10
	add	a,30h
	ld	(de),a
	djnz	vdispHLLoop
	ex	de,hl
	pop	de

fastVPuts:
	ld	(pencol),de
	RomCall(_vputs)
	ret

fastPuts:
	ld	(currow),de
	RomCall(_puts)
	ret

fastPutDigit:
	add	a,30h
fastPutmap:
	ld	(currow),hl
	RomCall(_putmap)
	ret

genRandom:
	di
	exx
	ld	c,a
	ex	af,af'
	ld	hl,0
randSeed =$+1
	ld	de,0001h
	ld	a,75
	ld	b,8
randomLoop:
	add	hl,hl
	jr	nc,$+3
	dec	hl
	rla
	jr	nc,$+6
	add	hl,de
	jr	nc,$+3
	dec	hl
	djnz	randomLoop
	ld	(randSeed),hl
	ld	d,b
	ld	e,h
	ld	h,b
	ld	l,b
	ld	b,c
	add	hl,de
	djnz	$-1
	ex	af,af'
	ld	a,h
	exx
	ret

pointHLind2A:
	add	a,a
pointHLindA:
	ld	d,0
	ld	e,a
	add	hl,de
	jp	_ldHLind

fastCopy:
	di
	ld	a,80h
	out	(10h),a
	ld	hl,screenBuf-12-(-(12*64)+1)
	ld	a,20h
	ld	c,a
	inc	hl
	dec	hl
fastCopyAgain:
	ld	b,64
	inc	c
	ld	de,-(12*64)+1
	out	(10h),a
	add	hl,de
	ld	de,6
fastCopyLoop:
	add	hl,de
	add	hl,de
	inc	de
	ld	a,(hl)
	out	(11h),a
	dec	de
	djnz	fastCopyLoop
	ld	a,c
	cp	2Ch
	jr	nz,fastCopyAgain
	ret

;===========================================================================
; * Text *
;===========================================================================

titleTxt:
	.db	"Z",0,"T",0,"R",0," I",0,"S",0," ",0,"8",0,"2"
byTxt:
	.db	"  1.3:TijlCJimmyMZTetris",0
heightTxt:
	.db	"Height",0
	.db	"  1 player ",0
	.db	"  2 players ",0
	.db	"Add ",0
	.db	"unscrambled"
linesTxt:
	.db	" lines",0
sendTxt:
	.db	"Send ",0
	.db	"2-4",0
	.db	"1-3",0
levelsTxt:
	.db	" 0 1 2 3 4",0
	.db	" 5 6 7 8 9",0
waitingTxt:
	.db	"* WAITING *",0
gameInfoTxt:
	.db	"Score",0
	.db	"Level",0
	.db	"Lines",0
pauseTxt:
	.db	"* PAUSE *",0
gameOverTxt:
	.db	" Game Over ",0
winTxt:
	.db	"  You Win  ",0
highScoreTxt:
	.db	"You have a high score!",0
	.db	"Enter your name",0

;===========================================================================
; * Data *
;===========================================================================

highScoreTable:
	.db	"1. ----------",0,0,0
	.db	"2. ----------",0,0,0
	.db	"3. ----------",0,0,0
patterns:			; Pattern for each level
	.db	$AA,$55,$AA,$55,$AA,$55,$AA,$55
	.db	$88,$FF,$22,$FF,$88,$FF,$22,$FF
	.db	$FF,$99,$99,$FF,$FF,$99,$99,$FF
	.db	$50,$D7,$14,$F7,$00,$F7,$14,$D7
	.db	$EA,$AA,$AE,$00,$57,$55,$75,$00
	.db	$FE,$AA,$AA,$AA,$28,$AA,$FE,$00
	.db	$55,$55,$AA,$AA,$55,$55,$AA,$AA
	.db	$80,$FE,$02,$FB,$08,$EF,$20,$BF
	.db	$EE,$EE,$EE,$00,$77,$77,$77,$00
	.db	$66,$CC,$99,$33,$66,$CC,$99,$33
	.db	$CC,$33,$CC,$33,$CC,$33,$CC,$33
	.db	$FE,$82,$BA,$AA,$BA,$82,$FE,$00
	.db	$CC,$CC,$33,$33,$CC,$CC,$33,$33
	.db	$FF,$AA,$FF,$AA,$FF,$AA,$FF,$AA
	.db	$7C,$FE,$7C,$00,$7C,$FE,$7C,$00
	.db	$FF,$EF,$47,$12,$B8,$FD,$FF,$FF
gaps:				; Gaps where the pattern shouldn't be shown
	.dw	screenBuf+25  \ .db 14
	.dw	screenBuf+217 \ .db 14
	.dw	screenBuf+409 \ .db 14
	.dw	screenBuf+601 \ .db 12
pieces:				; Compressed data of the pieces (28 pieces)
	.dw	%0001010110011000,%0110010100000100
	.dw	%0010000101011001,%0110101001010100
	.dw	%0001010110010000,%0010011001010100
	.dw	%0001010110011010,%0110010101001000
	.dw	%0101100101001000,%0101100101001000
	.dw	%0101100101001000,%0101100101001000
	.dw	%0001010110010100,%0001011001010100
	.dw	%0001011001011001,%0110010101001001
	.dw	%0000010101001001,%0010000101010100
	.dw	%0000010101001001,%0010000101010100
	.dw	%0001010101001000,%0001000001100101
	.dw	%0001010101001000,%0001000001100101
	.dw	%0001010110011101,%0111011001010100
	.dw	%0001010110011101,%0111011001010100
scoring:			; Score for each level
	.db	0,4,10,30,120
levelCnts:
	.db	120,108,96,84,75,66,57,51,45,39
	.db	33,29,25,21,18,15,12,9,6,3,1
characters:
	.db	"\"WRMH",0,0,"?",0,"VQLG",0,0
	.db	"!ZUPKFC",0," YTOJEB",0,0,"XSNIDA"
gameData:
score:	.dw	0
	.db	0
lines:	.db	0
board:	.db	255,255				; bottom
	.db	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0	; visible area
	.db	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0	;
	.db	0,0,0,0				; invisible area
	.db	0,0				; top
resume:	.db	0
stLevel:
	.db	0
level:	.db	0
stHeight:
	.db	0
gameOptions:
	.db	8
nxtPce:	.dw	0
cPce:	.dw	0
cPcePos:
	.dw	0
cPceRot:
	.db	0

;===========================================================================

_ldHLind:
 LD_HL_MHL
 ret
_HtimesL:
 ld b,h
 ld h,0
 ld d,h
 ld e,h
 ex de,hl
 or a
multloop:
 adc hl,de
 djnz multloop
 ret

.end